/*
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this code; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  */

/* Copyright (C) 2002 Ian Zimmerman <itz@speakeasy.org> */

static const char rcsid[] = "$Id: msc.c,v 1.2 2002/06/17 19:03:24 itz Exp $ ";

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <paths.h>
#include <fcntl.h>
#include <ggi/gii.h>
#include <ggi/gii-unix.h>
#include <limits.h>
#include <assert.h>

static int repeat_fd;
static int repeat_fd_opened = 0;

static int x, y;
unsigned char buttons = 0x87;

static unsigned button_map[] = { 4, 1, 2 };

int
repeat_init(void)
{
  static const char repeat_path[] = _PATH_DEV "gim-msc";

  if (!repeat_fd_opened) {
    repeat_fd = open(repeat_path, O_RDWR|O_NONBLOCK);
    if (repeat_fd < 0) return -1;
    repeat_fd_opened = 1;
  } /*if*/

  x = y = 0; buttons = 0x87;
}

int
repeat_event (gii_event* pev)
{
  int dx, dy, dx_half, dy_half;
  int button;
  union {
    unsigned char uch[5];
    char ch[5];
  } u;
  
  switch (pev->any.type) {
  case evPtrAbsolute:
    dx = pev->pmove.x - x;
    dy = pev->pmove.y - y;
    if (dx == 0 && dy == 0) return 0;
    x = pev->pmove.x;
    y = pev->pmove.y;
    break;
  case evPtrRelative:
    dx = pev->pmove.x;
    dy = pev->pmove.y;
    if (dx == 0 && dy == 0) return 0;
    x += dx;
    y += dy;
    break;
  case evPtrButtonRelease:
    button = pev->pbutton.button;
    if (button <= 3) {
      buttons |= button_map[button - 1];
    } else {
      return 0;
    } /*if*/
    dx = dy = 0;
    break;
  case evPtrButtonPress:
    button = pev->pbutton.button;
    if (button <= 3) {
      buttons &= ~button_map[button - 1];
    } else {
      return 0;
    } /*if*/
    dx = dy = 0;
    break;
  default:
    assert(0);
  } /*switch*/

  dx_half = dx >> 1; dy_half = - (dy >> 1);
  u.uch[0] = buttons;
  u.ch[1] = dx_half; u.ch[3] = dx - dx_half;
  u.ch[2] = dy_half; u.ch[4] = -dy - dy_half;

  return write(repeat_fd, &u, 5);
  
}
